@@ -6,7 +6,8 @@ import asyncio
 import datetime
 import itertools
 import logging
-from abc import ABC
+import time
+from abc import ABC, abstractmethod
 from collections import OrderedDict
 from collections.abc import Iterable
 from typing import Any, Awaitable, Callable, Dict
@@ -227,12 +228,15 @@ class EventMedia:
 class EventMediaStore(ABC):
     """Interface for external storage."""
 
+    @abstractmethod
     async def async_load(self) -> dict | None:
         """Load data."""
 
+    @abstractmethod
     async def async_save(self, data: dict) -> None:
         """Save data."""
 
+    @abstractmethod
     def get_media_key(self, device_id: str, event: ImageEventBase) -> str:
         """Return the filename to use for the device and event."""
 
@@ -296,19 +300,19 @@ class InMemoryEventMediaStore(EventMediaStore):
 
     async def async_load_media(self, media_key: str) -> bytes | None:
         """Load media content."""
-        DIAGNOSTICS.increment("load_media")
-        return self._media.get(media_key)
+        with DIAGNOSTICS.timer("load_media"):
+            return self._media.get(media_key)
 
     async def async_save_media(self, media_key: str, content: bytes) -> None:
         """Remove media content."""
-        DIAGNOSTICS.increment("save_media")
-        self._media[media_key] = content
+        with DIAGNOSTICS.timer("save_media"):
+            self._media[media_key] = content
 
     async def async_remove_media(self, media_key: str) -> None:
         """Remove media content."""
-        DIAGNOSTICS.increment("remove_media")
-        if media_key in self._media:
-            del self._media[media_key]
+        with DIAGNOSTICS.timer("remove_media"):
+            if media_key in self._media:
+                del self._media[media_key]
 
 
 class EventMediaModelItem:
@@ -529,10 +533,10 @@ class EventMediaManager:
             self._lock = asyncio.Lock()
         async with self._lock:
             event_data = await self._async_load()
-            _LOGGER.info("Checking cache size %s", len(event_data))
+            _LOGGER.debug("Checking cache size %s", len(event_data))
             if len(event_data) <= self._cache_policy.event_cache_size:
                 return
-            _LOGGER.info(
+            _LOGGER.debug(
                 "Expiring cache %s", self._cache_policy.event_cache_expire_count
             )
             # Bulk pop items
@@ -839,14 +843,19 @@ class EventMediaManager:
         if not event_sessions:
             return
         _LOGGER.debug("Event Update %s", event_sessions.keys())
+        recv_latency_ms = int((time.time() - event_message.timestamp.timestamp()) * 100)
 
         # Notify traits to cache most recent event
         pairs = list(event_sessions.items())
         for event_session_id, event_dict in pairs:
             supported = False
             for event_name, event in event_dict.items():
+                if not event.is_expired:
+                    self._diagnostics.elapsed(event_name, recv_latency_ms)
+                else:
+                    self._diagnostics.elapsed(f"{event_name}_expired", recv_latency_ms)
                 if not (trait := self._event_trait_map.get(event_name)):
-                    self._diagnostics.increment("event.unsupported_trait")
+                    self._diagnostics.increment(f"event.unsupported.{event_name}")
                     _LOGGER.debug("Unsupported event trait: %s", event_name)
                     continue
                 supported = True
@@ -860,23 +869,22 @@ class EventMediaManager:
         # to downstream subscribers the first time they are seen to avoid
         # firing on updated event threads multiple times.
         suppress_keys: set[str] = set({})
-        valid_events = 0
+        notifiable_events: dict[str, int] = {}  # Notifiable events for each session
         for event_session_id, event_dict in event_sessions.items():
-
             # Track all related events together with the same session
             if model_item := await self._async_load_item(event_session_id):
                 self._diagnostics.increment("event.update")
                 # Update the existing event session with new/updated events. Only
                 # new events are published.
                 suppress_keys |= set(model_item.events.keys())
-                valid_events += len(
+                notifiable_events[event_session_id] = len(
                     set(event_dict.keys()) - set(model_item.events.keys())
                 )
                 model_item.events.update(event_dict)
             else:
                 self._diagnostics.increment("event.new")
                 # A new event session
-                valid_events += len(event_dict.keys())
+                notifiable_events[event_session_id] = len(event_dict.keys())
                 model_item = EventMediaModelItem(
                     event_session_id,
                     event_dict,
@@ -889,10 +897,12 @@ class EventMediaManager:
 
             if self._support_fetch and self._cache_policy.fetch:
                 self._diagnostics.increment("event.fetch")
+                failure = False
                 try:
                     await self._fetch_media(model_item)
                 except GoogleNestException as err:
                     self._diagnostics.increment("event.fetch_error")
+                    failure = True
                     _LOGGER.warning(
                         "Failure when pre-fetching event '%s': %s",
                         event.event_session_id,
@@ -901,6 +911,19 @@ class EventMediaManager:
                 # Update any new media keys
                 await self._async_update_item(model_item)
 
+                # If no media was fetched, then pause until we get a message that contains
+                # media or the thread ends. If we attempted then failed, then pass on since
+                # the event could have been ready to go.
+                if (
+                    model_item.any_media_key is None
+                    and not event_message.is_thread_ended
+                    and not failure
+                ):
+                    _LOGGER.debug(
+                        "Skipping notification without media; Will deliver later"
+                    )
+                    notifiable_events[event_session_id] = 0
+
         await self._expire_cache()
 
         if not self._callback:
@@ -908,9 +931,14 @@ class EventMediaManager:
         # Notify any listeners about the arrival of a new event
         if suppress_keys:
             event_message = event_message.omit_events(suppress_keys)
-        if valid_events > 0:
+        if any(notifiable_events.values()):
+            _LOGGER.debug("Message contains notifiable events: %s", notifiable_events)
             self._diagnostics.increment("event.notify")
             await self._callback(event_message)
+        else:
+            _LOGGER.debug(
+                "Message did not contain notifiable events: %s", notifiable_events
+            )
 
     @property
     def active_event_trait(self) -> EventImageGenerator | None:
