@@ -7,6 +7,7 @@ import enum
 import json
 import logging
 import re
+import time
 from abc import ABC, abstractmethod
 from typing import Awaitable, Callable, Optional
 
@@ -250,7 +251,6 @@ class DefaultSubscriberFactory(AbstractSubscriberFactory):
         def callback_wrapper(message: pubsub_v1.subscriber.message.Message) -> None:
             if loop.is_closed():
                 return
-            assert async_callback
             future: concurrent.futures.Future = asyncio.run_coroutine_threadsafe(
                 async_callback(message), loop
             )
@@ -511,13 +511,22 @@ class GoogleNestSubscriber:
         self, message: pubsub_v1.subscriber.message.Message
     ) -> None:
         """Handle a received message."""
-        DIAGNOSTICS.increment("message_received")
         payload = json.loads(bytes.decode(message.data))
         event = EventMessage(payload, self._auth)
+        recv = time.time()
+        latency_ms = int((recv - event.timestamp.timestamp()) * 1000)
+        DIAGNOSTICS.elapsed("message_received", latency_ms)
         # Only accept device events once the Device Manager has been loaded.
         # We are ok with missing messages on startup since the device manager
-        # will do a live read .
-        if self._device_manager_task and self._device_manager_task.done():
+        # will do a live read. This checks for an exception to avoid throwing
+        # inside the pubsub callback and further weding the pubsub client library.
+        if (
+            self._device_manager_task
+            and self._device_manager_task.done()
+            and not self._device_manager_task.exception()
+        ):
             await self._device_manager_task.result().async_handle_event(event)
-        message.ack()
-        DIAGNOSTICS.increment("message_acked")
+            message.ack()
+
+        ack_latency_ms = int((time.time() - recv) * 1000)
+        DIAGNOSTICS.elapsed("message_acked", ack_latency_ms)
